/******************************************************************************
 *
 * Copyright (c) 1999 Palm Computing, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: Starter.cpp
 *
 *****************************************************************************/

#include <PalmOS.h>
#include "DBSLibFuserRsc.h"
#include "DBSLibFuser.h"
#include "DBSLib.h"
#include "AFFuserGlue.h"
#include "Debug.h"

#define dbsFuserAppLaunchGetAPI   32767
#define dbsFuserAppLaunchCreateDB 32768

/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;

/*
 *  Structure for marshalling data to AppForge
 *  From DBSLibGetAPI call
 */
typedef struct DBSLibAPIStruct
{
  Int16 nMajor;        /* OUT */
  Int16 nMinor;        /* OUT */
  UInt32 nResult;      /* OUT */
} DBSLibAPIStruct;

/* Corresponding VB type:
Public Type LibAPIStruct
   nMajor As Integer
   nMinor As Integer
   nResult As Long
End Type
*/

/*
 *  Structure for marshalling data to AppForge
 *  From DBSLibParseSchema & DBSLibGetDBInfo
 *  calls
 */
typedef struct CreateDBStruct
{
    UInt32 XMLString;  /* IN  */
    Boolean bDeleteDB;  /* IN  */
    UInt32 dbName;      /* OUT */
    Int16 nNumRecords;  /* OUT */
    Int16 nNumFields;   /* OUT */
    UInt32 Schema;     /* OUT */
    UInt32 nResult;     /* OUT */
} CreateDBStruct;

/* Corresponding VB type:
Type CreateDBStruct
    XMLString As String
    bDeleteDB As Boolean
    dbName As String
    nNumRecords As Integer
    nNumFields As Integer
    Schema As String
    nResult As Long
End Type
*/



/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/

//Structure to maintain information about a shared library
typedef struct ShLibInfoType
{
   //DBSLib reference number
   UInt16 nLibRefNum;   
   //Set to true if DBSLib was loaded by another
   //application, so we won't try to unload it
   Boolean bLibWasPreloaded;
   UInt32 nClientContext;
} ShLibInfoType;

typedef ShLibInfoType *ShLibInfoTypePtr;

//This global is only for use as a driver
//Use as an AppForge Fuser requires local variables!
static MemHandle g_hShLibInfo;


#define XML_STRING "<Database Name=Employee Type=DATA Creator=AFLD>" \
                        "<Schema>" \
                           "<Field>" \
                              "<Num>0</Num>" \
                              "<Name>FirstName</Name>" \
                              "<Type>6</Type>" \
                              "<Length>0</Length>" \
                           "</Field>" \
                           "<Field>" \
                              "<Num>1</Num>" \
                              "<Name>LastName</Name>" \
                              "<Type>6</Type>" \
                              "<Length>0</Length>" \
                           "</Field>" \
                           "<Field>" \
                              "<Num>2</Num>" \
                              "<Name>Phone</Name>" \
                              "<Type>6</Type>" \
                              "<Length>0</Length>" \
                           "</Field>" \
                           "<Field>" \
                              "<Num>3</Num>" \
                              "<Name>Email</Name>" \
                              "<Type>6</Type>" \
                              "<Length>0</Length>" \
                           "</Field>" \
                           "<Field>" \
                              "<Num>4</Num>" \
                              "<Name>ID</Name>" \
                              "<Type>5</Type>" \
                              "<Length>0</Length>" \
                           "</Field>" \
                        "</Schema>" \
                        "<Data>" \
                           "<Record>" \
                              "<FirstName>Bob H.</FirstName>" \
                              "<LastName>Smith</LastName>" \
                              "<Phone>(703) 555-1212</Phone>" \
                              "<Email>bob_s@company.com</Email>" \
                              "<ID>335</ID>"\
                           "</Record>" \
                           "<Record>" \
                              "<FirstName>Lucy D.</FirstName>" \
                              "<LastName>Anderson</LastName>" \
                              "<Phone>(703) 555-2121</Phone>" \
                              "<Email>landerson@company.com</Email>" \
                              "<ID>226</ID>" \
                           "</Record>" \
                           "<Record>" \
                              "<FirstName>Jane</FirstName>" \
                              "<LastName>Williams</LastName>" \
                              "<Phone>(703) 222-5151</Phone>" \
                              "<Email>janew@company.com</Email>" \
                              "<ID>23</ID>" \
                           "</Record>" \
                        "</Data>" \
                     "</Database>"

/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator				 'DBSD'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01

/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
static Err LoadShLib(Char *pszLibName, UInt32 nDBType, UInt32 nDBCreator, MemHandle *phShLibInfo);
static Err UnloadShLib(MemHandle hShLibInfo);

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean bHandled = false;
	
	return bHandled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
		{
		
      case ctlSelectEvent:
		   switch (eventP->data.ctlSelect.controlID)
		   {
		   
            case MainGetAPIButton:
		      {
		         Err retVal;
		         Char szTemp[256];
		         UInt32 nVersion;
		         ShLibInfoTypePtr pShLibInfo;
		         
		         pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(g_hShLibInfo);
               retVal = DBSLibGetAPIVersion(pShLibInfo->nLibRefNum, &nVersion);
               MemPtrUnlock(pShLibInfo);
               
               if(retVal == errNone)
               {
                  StrPrintF(szTemp, "Lib version = %d.%d",
                                     sysGetROMVerMajor(nVersion),
                                     sysGetROMVerMinor(nVersion));

               }
               else
                  StrCopy(szTemp, "Call to GetDBAPIVersion failed!");         

               FrmCustomAlert(InfoAlert, szTemp, "", "");        
  
		      }
		      break;
		   
		   
            case MainCreateDBButton:
            {
            
               Err retVal;   
               Char* pszXML = XML_STRING;
               ShLibInfoTypePtr pShLibInfo;
		         
		         pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(g_hShLibInfo);

               TRACE_TL0("Calling DBSLibParseSchema");
               retVal = DBSLibParseSchema(pShLibInfo->nLibRefNum, pShLibInfo->nClientContext, pszXML, true);
               MemPtrUnlock(pShLibInfo);
               
               if(retVal == errNone)
                  FrmCustomAlert(InfoAlert, "DBSLibParseSchema succeeded.", "", "");
               else
                  FrmCustomAlert(InfoAlert, "DBSLibParseSchema failed.", "", "");
               handled = true;
            }
		      break;
		      
		      case MainGetDBInfoButton:	      
		      {
		         Char szDBName[DB_NAME_LEN];
		         Char szSchema[DB_SCHEMA_LEN];
		         Int16 nNumRecords;
		         Int16 nNumFields;
		         Err retVal;
		         ShLibInfoTypePtr pShLibInfo;
		         
		         pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(g_hShLibInfo);
		         
		         TRACE_TL0("Calling DBSLibGetDBInfo");
		         retVal = DBSLibGetDBInfo(pShLibInfo->nLibRefNum, pShLibInfo->nClientContext, szDBName, &nNumRecords, &nNumFields, szSchema);
		         MemPtrUnlock(pShLibInfo);
		         TRACE_TL0("Database:");
		         TRACE_TL1("  Num records: %d", nNumRecords);
		         TRACE_TL1("   Num fields: %d", nNumFields);
		         TRACE_TL1("       Schema: %s", szSchema);
		         {
		            char szResult[128];
		            char szTemp[128];
		            
                   if(retVal == errNone)
                   {
                      
                      StrPrintF(szTemp, "Num recs = %d\n", nNumRecords);
                      StrCopy(szResult, szTemp);
                      StrPrintF(szTemp, "Num fields = %d\n", nNumFields);
                      StrCat(szResult, szTemp);
                      StrPrintF(szTemp, "Schema = %s\n", szSchema);
                      StrCat(szResult, szTemp);
                      
                   }
                   else
                      StrCopy(szResult, "DBSLibGetDBInfo failed.");
                      
                      FrmCustomAlert(InfoAlert, szResult, "", "");
                          
               }
               
		          
		      }
		      break;
		      		   
		   }
		break; // case ctlSelectEvent		
		
		
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
	   break;


		default:
	   break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
   StarterPreferenceType prefs;
   UInt16 prefsSize;
   Err retVal = errNone;
   ShLibInfoTypePtr pShLibInfo;

   // Read the saved preferences / saved-state information.
   prefsSize = sizeof(StarterPreferenceType);
   if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
      noPreferenceFound)
   {
   }


   retVal = LoadShLib(dbsLibName, dbsLibTypeID, dbsLibCreatorID, &g_hShLibInfo);
		   
   if(retVal != errNone)
      return retVal;
    
    
   pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(g_hShLibInfo);
      
   //Open the library

   TRACE_TL0("Opening library"); 
   retVal = DBSLibOpen(pShLibInfo->nLibRefNum, &(pShLibInfo->nClientContext));

   MemPtrUnlock(pShLibInfo);
			
   return retVal;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
   StarterPreferenceType prefs;
   ShLibInfoTypePtr pShLibInfo;
 
	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();

   // Close the library
   pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(g_hShLibInfo);
   DBSLibClose(pShLibInfo->nLibRefNum, pShLibInfo->nClientContext);
   MemPtrUnlock(pShLibInfo);
      
   UnloadShLib(g_hShLibInfo);
			

	
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err errVal;
	
   TRACE_INIT();
   
   TRACE_TL0("StarterPalmMain: IN");
      
	switch (cmd)
	{
	   //For use as a driver
	   //Compile with type 'appl'
		case sysAppLaunchCmdNormalLaunch:
		{
		   TRACE_TL0("case sysAppLaunchCmdNormalLaunch");
		   		   		   
			errVal = AppStart();
			if(errVal != errNone) 
				return errVal;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
						
		}
	   break;


      //For use as an AppForge Fuser
      //Compile with type as 'FUSR'
      case dbsFuserAppLaunchGetAPI:
      {
         MemHandle hShLibInfo;
         UInt32 nVersion;
         ShLibInfoTypePtr pShLibInfo;
         
         TRACE_TL0("case dbsFuserAppLaunchGetAPI");
       
         errVal = LoadShLib(dbsLibName, dbsLibTypeID, dbsLibCreatorID, &hShLibInfo);
    
         if(errVal == errNone)
         {
            pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(hShLibInfo);            
            errVal = DBSLibGetAPIVersion(pShLibInfo->nLibRefNum, &nVersion);
            MemPtrUnlock(pShLibInfo);
            
            if(errVal == errNone)
            {
               
               ((DBSLibAPIStruct *)cmdPBP)->nMajor = sysGetROMVerMajor(nVersion); 
               ((DBSLibAPIStruct *)cmdPBP)->nMinor = sysGetROMVerMinor(nVersion);
               
               ((DBSLibAPIStruct *)cmdPBP)->nResult = 0; 
               
            }
            else
            {
               ((DBSLibAPIStruct *)cmdPBP)->nResult = 1;  //GetAPIVersion failed
            }
            
            UnloadShLib(hShLibInfo);
         
         }
         else
         {
            TRACE_TL0("Could not load shared library");
            ((DBSLibAPIStruct *)cmdPBP)->nResult = 1;  //LoadShLib failed

         }
         
         errVal = errNone; //Reset the error - afPalmOS.CallApp succeeded
           
      }
      break;


      //For use as an AppForge Fuser
      //Compile with type as 'FUSR'
      case dbsFuserAppLaunchCreateDB:
      {
         MemHandle hXMLString;
         Boolean bDeleteDB;
         MemHandle hdbName;
         Int16 nNumRecords;
         Int16 nNumFields;
         MemHandle hSchema;
         
	     MemHandle hShLibInfo;
	     ShLibInfoTypePtr pShLibInfo;
	     Char *pXMLString;
	     Char *pdbName;
	     Char *pSchema;
	     Char szDBName[DB_NAME_LEN];
	     Char szSchema[DB_SCHEMA_LEN];
	  
         TRACE_TL0("case dbsDriverAppLaunchCreateDB");
         
         errVal = LoadShLib(dbsLibName, dbsLibTypeID, dbsLibCreatorID, &hShLibInfo);
  
         if(errVal == errNone)
         {  
            pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(hShLibInfo);
  
            //Open the library
            TRACE_TL0("Opening library"); 
            errVal = DBSLibOpen(pShLibInfo->nLibRefNum, &(pShLibInfo->nClientContext));
            
            TRACE_TL0("Opened library");
        
            if(errVal == errNone)
            {

               hXMLString = NULL;
               hXMLString = AFStringToMemHandle(((CreateDBStruct *)cmdPBP)->XMLString);
               
               if(hXMLString != NULL)
                  pXMLString = (Char *)MemHandleLock(hXMLString);
                                 
               bDeleteDB = ((CreateDBStruct *)cmdPBP)->bDeleteDB;
               
               if(bDeleteDB)
               {
                  bDeleteDB = true;
                  TRACE_TL0("bDeleteDB = true");
               }
               else
               {
                  bDeleteDB = false;
                  TRACE_TL0("bDeleteDB = false");
               }
               
               errVal = DBSLibParseSchema(pShLibInfo->nLibRefNum, pShLibInfo->nClientContext, pXMLString, bDeleteDB);
         
               
               MemPtrUnlock(pXMLString);
            
               if(errVal == errNone)
                  errVal = DBSLibGetDBInfo(pShLibInfo->nLibRefNum, pShLibInfo->nClientContext, szDBName, &nNumRecords, &nNumFields, szSchema);

               if(errVal == errNone)
               {
                  
                  // --- Assign the database name ---
                  //Remove old string               
                  hdbName = AFStringToMemHandle(((CreateDBStruct *)cmdPBP)->dbName);
                  if(hdbName != NULL)
                  {
                     TRACE_TL0("Freeing dbName");
                     MemHandleFree(hdbName);
                  }              
                              
                  hdbName = MemHandleNew(DB_NAME_LEN);
                  pdbName = MemHandleLock(hdbName);
                  StrCopy(pdbName, szDBName);
               
                  //Assign new string
                  errVal = MemHandleToAFString(hdbName, &(((CreateDBStruct *)cmdPBP)->dbName));
                               
                  MemPtrUnlock(pdbName);
            
            
                  // --- Assign the database schema ---
                  //Remove old string
                  hSchema = AFStringToMemHandle(((CreateDBStruct *)cmdPBP)->Schema);
                  
                  if(hSchema != NULL)  //Remove old string
                  {
                     TRACE_TL0("Freeing Schema");
                     MemHandleFree(hSchema);
                  }
               
                  hSchema = MemHandleNew(DB_SCHEMA_LEN);
                  pSchema = MemHandleLock(hSchema);
                  StrCopy(pSchema, szSchema);
                                 
                  //Assign new string
                  errVal = MemHandleToAFString(hSchema, &(((CreateDBStruct *)cmdPBP)->Schema));

                  MemPtrUnlock(pSchema);
               
               
                  // --- Assign the number of records ---
                  ((CreateDBStruct *)cmdPBP)->nNumRecords = nNumRecords;
               
               
                  // --- Assign the number of fields ---
                  ((CreateDBStruct *)cmdPBP)->nNumFields = nNumFields;
              
                  //DBSLibParseSchema & DBSLibDBGetInfo succeeded
                  ((CreateDBStruct *)cmdPBP)->nResult = 0;
                  
               }
               else
               {
                  ((CreateDBStruct *)cmdPBP)->nResult = 1; //DBSLibParseSchema or
                                                           //DBSLibDBGetInfo failed
               }
               
           
               // Close the library
	            DBSLibClose(pShLibInfo->nLibRefNum, pShLibInfo->nClientContext);
            
            }
            else
            {
               TRACE_TL0("Could not open library");
               ((CreateDBStruct *)cmdPBP)->nResult = 1;  //DBSLibOpen failed
            }
                    
            MemPtrUnlock(pShLibInfo);
            UnloadShLib(hShLibInfo);
	         
         }
         else
         {
            TRACE_TL0("Could not load shared library");
            ((CreateDBStruct *)cmdPBP)->nResult = 1;  //LoadShLib failed
         }
         
         
         //Reset the error - afPalmOS.CallApp succeeded
         errVal = errNone;
      }
      break;
      
      default:
		break;

	}
	
	TRACE_TL0("StarterPalmMain: OUT");
	
   TRACE_CLOSE();

	return errVal;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return StarterPalmMain(cmd, cmdPBP, launchFlags);
}


/************************************************************
 *
 *  FUNCTION: LoadShLib
 *
 *  DESCRIPTION: Finds and loads a shared libary.
 *
 *
 *  PARAMETERS: pszLibName
 *              Name of the shared libary (database) to find and load.
 *              
 *              nDBType
 *              Type of database to load .
 *
 *              nDBCreator
 *              Creator of database to load
 *
 *              phShLibInfo
 *              Upon success, filled with a handle to a ShLibInfoType
 *              structure.
 *
 *************************************************************/
Err LoadShLib(Char *pszLibName, UInt32 nDBType, UInt32 nDBCreator, MemHandle *phShLibInfo)
{
   Err retVal;
   
   MemHandle hShLibInfo;
   ShLibInfoTypePtr pShLibInfo;
   
   TRACE_TL0("LoadShLib: IN");
   
   hShLibInfo = MemHandleNew(sizeof(ShLibInfoType));
   
   if(hShLibInfo == NULL)
      return dbsLibFuserErrMemory;
      
   pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(hShLibInfo);
   
   //Load the DBSLib library
   retVal = SysLibFind(pszLibName, &(pShLibInfo->nLibRefNum));

	if(retVal == errNone)
	{
	   TRACE_TL0("Library HAS already been loaded by another application.");
	   pShLibInfo->bLibWasPreloaded = true;
	}
	else
	{
      //The library has not been already been loaded by another application.
      //We need to load ourselves
      TRACE_TL0("Library HAS NOT already been loaded by another application.");
      pShLibInfo->bLibWasPreloaded = false;
      retVal = SysLibLoad(nDBType, nDBCreator, &(pShLibInfo->nLibRefNum));
      
      TRACE_TL1("Lib ref num = %d", pShLibInfo->nLibRefNum);

 	}
	
  *phShLibInfo = hShLibInfo;
  
  MemPtrUnlock(pShLibInfo);

  TRACE_TL0("LoadShLib: OUT");

   return retVal;
}

/************************************************************
 *
 *  FUNCTION: UnloadShLib
 *
 *  DESCRIPTION:  Unloads a shared library.
 *
 *  PARAMETERS: hShLibInfo
 *              Handle to a ShLibInfoType struct which describes
 *              the library to be unloaded.
 *
 *************************************************************/
Err UnloadShLib(MemHandle hShLibInfo)
{

   
   Err retVal = errNone;
   ShLibInfoTypePtr pShLibInfo;

   TRACE_TL0("UnloadShLib: IN");

   
   pShLibInfo = (ShLibInfoTypePtr)MemHandleLock(hShLibInfo);
   	
   //Uninstall the library
   if(pShLibInfo->bLibWasPreloaded == false && pShLibInfo->nLibRefNum != sysInvalidRefNum)
   {
      TRACE_TL0("Library HAS NOT already been loaded by another application.");
      retVal = SysLibRemove(pShLibInfo->nLibRefNum);
      ErrFatalDisplayIf(retVal != errNone, "DBSLibDriver: Error uninstalling DBSLib.");
      pShLibInfo->nLibRefNum = sysInvalidRefNum;    
   }	

   MemPtrFree(pShLibInfo); 

   TRACE_TL0("UnloadShLib: OUT");

   return retVal;
}